import { MessageTransmissionTransmitter } from "./msg.transmission.transmitter";
import { MessageTransmissionReceiver } from "./msg.transmission.receiver";
import { AdapterManager } from "../adapters/adapter.manager";
import { v4 as uuidv4 } from 'uuid'
import { MessageTransmissionRequestResponse } from "./msg.transmission.request-response";
import { filter, Observable, Observer, Subject, Unsubscribable } from "rxjs";
import ConsoleLogger from "../utils/log.utils";
import { TransmitterAdapter } from "../adapters/adapter.transmitter"
import { ReceiverAdapter } from "../adapters/adapter.receiver"
import { MessageTransmissionManagerBase } from "../base/msg.transmission.manager.base";
import { ActorInterface, ActorProfile } from "../interface/actor.sample";
import { GeneralEvent, TransmissionInterface, TransportServiceInterface } from "../interface/interface";

export class MessageTransmissionManager extends MessageTransmissionManagerBase {
    private console: ConsoleLogger = new ConsoleLogger(`MessageTransmissionManager`, ['managers'])

    constructor(event: Subject<GeneralEvent<any>>, browserEnv?: boolean) {
        super()
        if (browserEnv) this.browserEnv = browserEnv
        this.console.log({ message: `Constructing self...` })
        this.event = event
        // Subscribe for adapterManager and it's relevent event
        this.adapterManager = new AdapterManager(event)
        this.adapterManager
    }

    public subscribeForTransmission(): Observable<GeneralEvent<TransmissionInterface>> {
        return new Observable((observer: Observer<GeneralEvent<TransmissionInterface>>) => {
            this.event.pipe(
                filter(event => event.event == 'New Client')
            ).subscribe(event => {
                // get all adapters for all the connection
                let transmission: TransmissionInterface | undefined = this.instantiateTransmissionComponents(event?.data?.clientId, this.event)
                if (transmission) {
                    observer.next({
                        id: uuidv4(),
                        type: `Transmission Event`,
                        event: 'New Transmission',
                        date: new Date(),
                        data: transmission
                    })
                }
            })
        })
    }

    protected instantiateTransmissionComponents(clientId: string, eventRef: Subject<GeneralEvent<any>>): TransmissionInterface {
        let receiverInstance: MessageTransmissionReceiver = new MessageTransmissionReceiver(clientId, eventRef)
        let transmitterInstance: MessageTransmissionTransmitter = new MessageTransmissionTransmitter(clientId, eventRef)
        let requestResponseInstance: MessageTransmissionRequestResponse = new MessageTransmissionRequestResponse(transmitterInstance, receiverInstance, eventRef)
        let transmissionObj: TransmissionInterface = {
            clientId: clientId,
            transmitter: transmitterInstance,
            receiver: receiverInstance,
            requestResponse: requestResponseInstance,
            event: this.event
        }

        return transmissionObj
    }

}


